package net.ibizsys.central.plugin.boot.core.runtime;

import com.alibaba.cloud.nacos.NacosConfigManager;
import com.alibaba.cloud.nacos.NacosConfigProperties;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.annotation.JSONField;
import com.alibaba.nacos.api.config.ConfigService;
import com.baomidou.dynamic.datasource.spring.boot.autoconfigure.DynamicDataSourceProperties;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.extern.slf4j.Slf4j;
import net.ibizsys.central.cloud.core.ServiceHubSettingBase;
import net.ibizsys.central.cloud.core.util.domain.DataSource;
import net.ibizsys.central.cloud.core.util.domain.DeploySystem;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.context.properties.NestedConfigurationProperty;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.yaml.snakeyaml.Yaml;

import javax.annotation.PostConstruct;
import java.util.*;

@Component
@ConfigurationProperties(BootServiceHubSetting.PREFIX)
@Slf4j
public class BootServiceHubSetting extends ServiceHubSettingBase {


	/**
	 * 数据项：服务网关前缀
	 */
	public final static String DATAID_SERVICEHUB_PREFIX = "servicehub-";

	/**
	 * 数据项：部署系统前缀
	 */
	public final static String DATAID_DEPLOYSYSTEM_PREFIX = "deploysystem-";
	
	/**
	 * 数据项：数据库实例前缀
	 */
	public final static String DATAID_DBINST_PREFIX = "dbinst-";

	@JsonIgnore
	@JSONField(serialize = false)
	@Autowired
	private NacosConfigManager nacosConfigManager;

	@JsonIgnore
	@JSONField(serialize = false)
	@Autowired
	NacosConfigProperties nacosConfigProperties;

	@JsonIgnore
	@JSONField(serialize = false)
	@Autowired
    BootDeploySystems bootDeploySystems;

	@JsonIgnore
	@JSONField(serialize = false)
	@Autowired
	DynamicDataSourceProperties dynamicDataSourceProperties;


	private List<String> loadDeploySystems;
	public List<String> getLoadDeploySystems() {
		return loadDeploySystems;
	}
	public void setLoadDeploySystems(List<String> loadDeploySystems) {
		this.loadDeploySystems = loadDeploySystems;
	}

	@JsonProperty("systemsettings")
	@JSONField(name = "systemsettings")
	@NestedConfigurationProperty
	private BootDeploySystems.BootDeploySystemSetting systemSettings;
	public BootDeploySystems.BootDeploySystemSetting getSystemSettings() {
		return systemSettings;
	}
	public void setSystemSettings(BootDeploySystems.BootDeploySystemSetting systemSettings) {
		this.systemSettings = systemSettings;
	}


	@PostConstruct
	public void postConstruct() {
		loadConfig();
	}

	private boolean bLoadConfig = false;


	public synchronized void loadConfig() {
		if (bLoadConfig) {
			return;
		}
		try {
			onLoadConfig();
			bLoadConfig = true;
		} catch (Exception e) {
			throw new RuntimeException(e.getMessage(), e);
		}
	}

	protected void onLoadConfig() throws Exception {

		ConfigService configService = nacosConfigManager.getConfigService();

		Map<String, Object> commonSettings = null;
		List<DataSource> dataSourceList = new ArrayList<DataSource>();

		// 本方法启动的时候获取内容
		String content = configService.getConfig(DATAID_SERVICEHUB_PREFIX + this.getId(), this.nacosConfigProperties.getGroup(), 5000);
		if(StringUtils.hasLength(content)) {
			Yaml yaml = new Yaml();
			Map config = yaml.loadAs(content, Map.class);

			Object value = config.get("enableappgateway");
			if (value != null) {
				this.setEnableAppGateway(value.toString().equalsIgnoreCase("true"));
			}
			value = config.get("tempfolder");
			if (value != null) {
				this.setTempFolder(value.toString());
			}
			value = config.get("filefolder");
			if (value != null) {
				this.setFileFolder(value.toString());
			}
			value = config.get("fontfolder");
			if (value != null) {
				this.setFontFolder(value.toString());
			}
			value = config.get("reportfolder");
			if (value != null) {
				this.setReportFolder(value.toString());
			}
			value = config.get("node");
			if (value != null) {
				this.setNode(Long.valueOf(value.toString()));
			}


			if (config.get("systemsettings") instanceof Map) {
				commonSettings = (Map) config.get("systemsettings");
			}

			Object objDeploySystems = config.get("deploysystems");
			if (objDeploySystems instanceof List)
				loadDeploySystems = (List) objDeploySystems;


			Object objDataSources = config.get("datasources");
			if (objDataSources instanceof Map) {

				Map<String, Object> map = (Map<String, Object>) objDataSources;
				for (Map.Entry<String, Object> entry : map.entrySet()) {
					DataSource dataSource = new DataSource();
					Map<String, Object> source = (Map<String, Object>) entry.getValue();
					dataSource.putAll(source);
					//进一步处理属性，这些属性将完全复制之数据源配置
					for (Map.Entry<String, Object> item : source.entrySet()) {
						String strKey = item.getKey().replace("-", "");
						if (strKey.equals(item.getKey())) {
							continue;
						}
						dataSource.set(strKey, item.getValue());
					}


					dataSource.setDataSourceId(entry.getKey());
					if (ObjectUtils.isEmpty(dataSource.getDriverClassName())) {
						dataSource.setDriverClassName((String) dataSource.get("driver-class-name"));
					}
					if (ObjectUtils.isEmpty(dataSource.getJdbcUrl())) {
						dataSource.setJdbcUrl((String) dataSource.get("url"));
					}
					dataSourceList.add(dataSource);
				}
				this.setDataSources(dataSourceList);
			}
		}
		else {
			log.warn(String.format("无法获取远程配置[%1$s]，使用本地配置文件", DATAID_SERVICEHUB_PREFIX + this.getId()));
			if(!ObjectUtils.isEmpty(this.systemSettings))
				commonSettings= JSON.parseObject(JSON.toJSONString(this.systemSettings), LinkedHashMap.class);


			dynamicDataSourceProperties.getDatasource().entrySet().forEach(entry->{
				DataSource dataSource = new DataSource();
				dataSource.setDataSourceId(getId()+"__"+(entry.getKey().equalsIgnoreCase("master")?"default":entry.getKey()));
				dataSource.setDriverClassName(entry.getValue().getDriverClassName());
				dataSource.setJdbcUrl(entry.getValue().getUrl());
				dataSource.setUsername(entry.getValue().getUsername());
				dataSource.setPassword(entry.getValue().getPassword());
				dataSourceList.add(dataSource);
			});
			this.setDataSources(dataSourceList);
		}

		if(!ObjectUtils.isEmpty(loadDeploySystems))
		{
			List<DeploySystem> deploySystemList = new ArrayList<DeploySystem>();
			for (Object item : loadDeploySystems) {
				if (ObjectUtils.isEmpty(item)) {
					continue;
				}
				DeploySystem deploySystem = new DeploySystem();
				String deploySystemConfig = configService.getConfig(DATAID_DEPLOYSYSTEM_PREFIX + item.toString(), this.nacosConfigProperties.getGroup(), 5000);
				if (StringUtils.hasLength(deploySystemConfig)) {
					Yaml yaml = new Yaml();
					Map map2 = yaml.loadAs(deploySystemConfig, Map.class);
					deploySystem.putAll(map2);
					deploySystem.setDeploySystemId(item.toString());
				}
				else if(bootDeploySystems.getDeploySystem(item.toString())!=null){
					deploySystem.putAll(JSON.parseObject(JSON.toJSONString(bootDeploySystems.getDeploySystem(item.toString())),LinkedHashMap.class));
				}
				else
					continue;
				// 合并设置
				if (commonSettings != null) {
					Map<String, Object> settings = new HashMap<String, Object>();
					settings.putAll(commonSettings);
					if (deploySystem.getSettings() != null) {
						settings.putAll(deploySystem.getSettings());
					}
					deploySystem.setSettings(settings);
				}
				deploySystemList.add(deploySystem);
			}
			this.setDeploySystems(deploySystemList);
		}

	}

}
